/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import javax.swing.JComponent;
import javax.vecmath.Matrix4f;

import de.javagl.swogl.geometry.Geometries;
import de.javagl.swogl.geometry.Geometry;

/**
 * Default implementation of a {@link SwoglComponent}
 */
final class DefaultSwoglComponent implements SwoglComponent
{
    /**
     * The Swing component contained in this SwoglComponent
     */
    private final JComponent component;
    
    /**
     * The {@link Texture} of this SwoglComponent
     */
    private final Texture texture;
    
    /**
     * The {@link Geometry} of this SwoglComponent
     */
    private final Geometry geometry;
    
    /**
     * The transform matrix of this SwoglComponent
     */
    private final Matrix4f transform;

    /**
     * Creates a new SwoglComponent that contains the given Swing component.
     * The geometry and texture of the SwoglComponent will have the 
     * size that is returned as the preferred size of the given component.
     * 
     * @param component The component
     */
    DefaultSwoglComponent(JComponent component)
    {
        this(component, 
            component.getPreferredSize().width, 
            component.getPreferredSize().height);
    }
    
    /**
     * Creates a new SwoglComponent that contains the given Swing component.
     * The geometry and texture of the SwoglComponent will have the 
     * given size.
     * 
     * @param component The component
     * @param w The width
     * @param h The height
     */
    DefaultSwoglComponent(JComponent component, int w, int h)
    {
        this(
            Geometries.create(w, h, 2, 2), 
            Textures.create(w, h), component);
    }
    
    /**
     * Create a new SwoglComponent that contains the given Swing component
     * and consists of the given geometry and texture.
     * 
     * @param geometry The geometry
     * @param texture The texture
     * @param component The component
     */
    DefaultSwoglComponent(
        Geometry geometry, Texture texture, JComponent component)
    {
        this.geometry = geometry;
        this.texture = texture;
        this.component = component;
     
        transform = new Matrix4f();
        transform.setIdentity();
    }
    
    
    @Override
    public JComponent getComponent()
    {
        return component;
    }

    @Override
    public Texture getTexture()
    {
        return texture;
    }
    
    @Override
    public Geometry getGeometry()
    {
        return geometry;
    }
    
    @Override
    public Matrix4f getTransform()
    {
        return new Matrix4f(transform);
    }
    
    @Override
    public void setTransform(Matrix4f transform)
    {
        this.transform.set(transform);
    }

    @Override
    public String toString()
    {
        return "DefaultSwoglComponent["+
            "component="+component+","+
            "geometry="+geometry+","+
            "texture="+texture+"]";
            //"transform="+transform+"]";
    }



}