/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import java.awt.Rectangle;
import java.awt.image.BufferedImage;

/**
 * Default implementation of a {@link Texture}.
 */
final class DefaultTexture implements Texture
{
    
    /**
     * The current dirty rectangle of this texture
     */
    private Rectangle dirtyRectangle = null;
    
    /**
     * The BufferedImage backing this texture.
     */
    private final BufferedImage image;
    
    /**
     * Creates a new Texture with the given size. The width and height will 
     * be clamped to be at least 1.
     * 
     * @param width The width of the texture
     * @param height The height of the texture
     */
    DefaultTexture(int width, int height)
    {
        width = Math.max(1, width);
        height = Math.max(1, height);
        image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        dirtyRectangle = new Rectangle(0,0,width,height);
    }

    @Override
    public BufferedImage getImage()
    {
        return image;
    }
    
    @Override
    public void addDirtyRectangle(Rectangle rectangle)
    {
        if (dirtyRectangle == null)
        {
            dirtyRectangle = new Rectangle(rectangle);
        }
        else
        {
            DirtyRectangleManager.merge(dirtyRectangle, 
                rectangle.x, rectangle.y, 
                rectangle.width, rectangle.height);
        }
    }
    
    @Override
    public Rectangle getDirtyRectangle()
    {
        if (this.dirtyRectangle == null)
        {
            return null;
        }
        return new Rectangle(this.dirtyRectangle);
    }
    
    @Override
    public void clearDirtyRectangle()
    {
        this.dirtyRectangle = null;
    }

    @Override
    public String toString()
    {
        return "DefaultTexture["+
            "image="+image+","+
            "dirtyRectangle="+dirtyRectangle+"]";
    }


}
