/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import java.awt.AWTEvent;
import java.awt.Component;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

/**
 * This class contains some utility methods for event dispatching
 */
class EventUtils
{
    /**
     * Dispatch the given event to all listeners of the given 
     * component.
     *  
     * @param component The component
     * @param mouseEvent The event to dispatch
     */
    static void dispatchToListeners(Component component, MouseEvent mouseEvent)
    {   
        for (MouseMotionListener m : component.getMouseMotionListeners())
        {
            EventUtils.dispatchMouseMotionEvent(mouseEvent, m);
        }
        for (MouseListener m : component.getMouseListeners())
        {
            EventUtils.dispatchMouseEvent(mouseEvent, m);
        }
        for (MouseWheelListener m : component.getMouseWheelListeners())
        {
            EventUtils.dispatchMouseWheelEvent(mouseEvent, m);
        }
    }
    
    
    /**
     * Will pass the given event to the given listener's methods, if the
     * event has the appropriate type
     * 
     * @param event The event to process
     * @param mouseWheelListener The listener to receive the event
     */
    static void dispatchMouseWheelEvent(
        AWTEvent event, MouseWheelListener mouseWheelListener)
    {
        if (event instanceof MouseEvent)
        {
            MouseEvent mouseEvent = (MouseEvent)event;
            switch (mouseEvent.getID())
            {
                case MouseEvent.MOUSE_WHEEL:
                    MouseWheelEvent mouseWheelEvent = 
                        (MouseWheelEvent)mouseEvent;
                    mouseWheelListener.mouseWheelMoved(mouseWheelEvent);
                    break;
            }
        }
    }
    
    /**
     * Will pass the given event to the given listener's methods, if the
     * event has the appropriate type
     * 
     * @param event The event to process
     * @param mouseMotionListener The listener to receive the event
     */
    static void dispatchMouseMotionEvent(
        AWTEvent event, MouseMotionListener mouseMotionListener)
    {
        if (event instanceof MouseEvent)
        {
            MouseEvent mouseEvent = (MouseEvent)event;
            switch (mouseEvent.getID())
            {
                case MouseEvent.MOUSE_MOVED:
                    mouseMotionListener.mouseMoved(mouseEvent);
                    break;

                case MouseEvent.MOUSE_DRAGGED:
                    mouseMotionListener.mouseDragged(mouseEvent);
                    break;
            }
        }
    }
    
    
    /**
     * Will pass the given event to the given listener's methods, if the
     * event has the appropriate type
     * 
     * @param event The event to process
     * @param mouseListener The listener to receive the event
     */
    static void dispatchMouseEvent(AWTEvent event, MouseListener mouseListener)
    {
        if (event instanceof MouseEvent)
        {
            MouseEvent mouseEvent = (MouseEvent)event;
            switch (mouseEvent.getID())
            {
                case MouseEvent.MOUSE_CLICKED:
                    mouseListener.mouseClicked(mouseEvent);
                    break;

                case MouseEvent.MOUSE_ENTERED:
                    mouseListener.mouseEntered(mouseEvent);
                    break;

                case MouseEvent.MOUSE_EXITED:
                    mouseListener.mouseExited(mouseEvent);
                    break;

                case MouseEvent.MOUSE_PRESSED:
                    mouseListener.mousePressed(mouseEvent);
                    break;

                case MouseEvent.MOUSE_RELEASED:
                    mouseListener.mouseReleased(mouseEvent);
                    break;
            }
        }
    }
    
    /**
     * Will pass the given event to the given listener's methods, if the
     * event has the appropriate type
     * 
     * @param event The event to process
     * @param keyListener The listener to receive the event
     */
    static void dispatchKeyEvent(AWTEvent event, KeyListener keyListener)
    {
        if (event instanceof KeyEvent)
        {
            KeyEvent keyEvent = (KeyEvent)event;
            switch (keyEvent.getID())
            {
                case KeyEvent.KEY_PRESSED:
                    keyListener.keyPressed(keyEvent);
                    break;

                case KeyEvent.KEY_RELEASED:
                    keyListener.keyReleased(keyEvent);
                    break;
            
                case KeyEvent.KEY_TYPED:
                    keyListener.keyTyped(keyEvent);
                    break;
            }
        }
    }
    
    
    /**
     * Static helper creating a short String for the given event
     * 
     * @param event The event
     * @return A short string describing the event
     */
    static String createInfoString(AWTEvent event)
    {
        StringBuffer str = new StringBuffer(80);
        str.append(event.getClass().getSimpleName()+" ");
        
        if (event instanceof MouseEvent)
        {
            int id = event.getID();
            switch(id) {
                case MouseEvent.MOUSE_PRESSED:
                    str.append("MOUSE_PRESSED");
                    break;
                case MouseEvent.MOUSE_RELEASED:
                    str.append("MOUSE_RELEASED");
                    break;
                case MouseEvent.MOUSE_CLICKED:
                    str.append("MOUSE_CLICKED");
                    break;
                case MouseEvent.MOUSE_ENTERED:
                    str.append("MOUSE_ENTERED");
                    break;
                case MouseEvent.MOUSE_EXITED:
                    str.append("MOUSE_EXITED");
                    break;
                case MouseEvent.MOUSE_MOVED:
                    str.append("MOUSE_MOVED");
                    break;
                case MouseEvent.MOUSE_DRAGGED:
                    str.append("MOUSE_DRAGGED");
                    break;
                case MouseEvent.MOUSE_WHEEL:
                    str.append("MOUSE_WHEEL");
                    break;
                default:
                    str.append("unknown type");
            }
            MouseEvent mouseEvent = (MouseEvent)event;
            str.append(
                " at "+mouseEvent.getX()+
                ","+ mouseEvent.getY()+
                " button "+mouseEvent.getButton());
            
        }
        if (event instanceof KeyEvent)
        {
            int id = event.getID();
            switch(id) {
                case KeyEvent.KEY_PRESSED:
                    str.append("KEY_PRESSED");
                    break;
                case KeyEvent.KEY_RELEASED:
                    str.append("KEY_RELEASED");
                    break;
                case KeyEvent.KEY_TYPED:
                    str.append("KEY_TYPED");
                    break;
                default:
                    str.append("unknown type");
            }
            KeyEvent keyEvent = (KeyEvent)event;
            str.append("  "+InputEvent.getModifiersExText(
                keyEvent.getModifiersEx()));
        }
        String s = String.valueOf(event.getSource());
        int index = s.indexOf("\n");
        if (index != -1)
        {
            s = s.substring(0, index)+"...";
        }
        str.append(" on "+s);
        return str.toString();
    }
    
    /**
     * Private constructor to prevent instantiation
     */
    private EventUtils()
    {
        
    }
    
}
