package de.javagl.swogl;

import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

/**
 * Utility methods for computing intersections
 */
class Intersection
{
    /**
     * A class for computing ray-triangle intersections
     */
    static class RayTriangle
    {
        // Temporary variables
        private static final float EPSILON = 1e-7f;
        private static final Vector3f tempEdge0 = new Vector3f();
        private static final Vector3f tempEdge1 = new Vector3f();
        private static final Vector3f tempVector3f0 = new Vector3f();
        private static final Vector3f tempVector3f1 = new Vector3f();
        private static final Vector3f tempVector3f2 = new Vector3f();
        
        /**
         * Computes the intersection position of a ray and a triangle
         * 
         * @param rayOrigin Origin of the ray
         * @param rayDirection The <strong>normalized</strong> 
         *     direction of the ray
         * @param p0 Point 0 of the triangle
         * @param p1 Point 1 of the triangle
         * @param p2 Point 2 of the triangle
         * @param result If there is an intersection, this afterwards
         * contains the intersection position (u,v,t) where (u,v) are 
         * coordinates describing the intersection point S as
         * S = u*(p1-p0)+v*(p2-p0) 
         * and t is the distance of the intersection point S along the ray.
         * The (u,v,t) tuple may be converted to barycentric coordinates
         * as follows:
         * bary.x = (1-u-v);
         * bary.y = u;
         * bary.z = v;
         * @return Whether there was an intersection
         */
        static boolean intersect(
            Tuple3f rayOrigin, Vector3f rayDirection, 
            Tuple3f p0, Tuple3f p1, Tuple3f p2, Tuple3f result)
        {
            tempEdge0.sub(p1, p0);
            tempEdge1.sub(p2, p0);
            tempVector3f0.cross(rayDirection, tempEdge1);
            float a = tempEdge0.dot(tempVector3f0);
            if (a > -EPSILON && a < EPSILON)
            {
                return false;
            }
            float b = 1.0f / a;
            tempVector3f1.sub(rayOrigin, p0);
            float u = b * tempVector3f1.dot(tempVector3f0);
            if (u < 0.0f || u > 1.0f)
            {
                return false;
            }
            tempVector3f2.cross(tempVector3f1, tempEdge0);
            float v = b * rayDirection.dot(tempVector3f2);
            if (v < 0.0f || v + u > 1.0f)
            {
                return false;
            }
            float t = b * tempEdge1.dot(tempVector3f2);
            result.set(u,v,t);
            return true;
        }

        /**
         * Private constructor to prevent instantiation
         */
        private RayTriangle()
        {
            
        }
    }
    
    
    /**
     * Private constructor to prevent instantiation
     */
    private Intersection()
    {
        
    }
}
