/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;
import java.nio.FloatBuffer;

import javax.vecmath.AxisAngle4f;
import javax.vecmath.Matrix4f;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;


/**
 * Matrix utility methods
 */
public class MatrixUtils
{
    /**
     * Creates and returns a scaling matrix.
     * 
     * @param s The scaling
     * @return The matrix
     */
    public static Matrix4f scale(float s)
    {
        Matrix4f result = identity();
        result.setScale(s);
        return result;
    }
    
    /**
     * Creates and returns a scaling matrix
     * 
     * @param x Scaling in x-direction
     * @param y Scaling in y-direction
     * @param z Scaling in z-direction
     * @return A matrix describing the specified scaling
     */
    public static Matrix4f scale(float x, float y, float z)
    {
        Matrix4f result = identity();
        result.setElement(0,0,x);
        result.setElement(1,1,y);
        result.setElement(2,2,z);
        return result;
    }
    
    
    /**
     * Creates and returns a rotation matrix.
     * 
     * @param angleRad The angle
     * @return The matrix
     */
    public static Matrix4f rotateX(double angleRad)
    {
        Matrix4f result = identity();
        result.rotX((float)angleRad);
        return result;
    }
    
    /**
     * Creates and returns a rotation matrix.
     * 
     * @param angleRad The angle
     * @return The matrix
     */
    public static Matrix4f rotateY(double angleRad)
    {
        Matrix4f result = identity();
        result.rotY((float)angleRad);
        return result;
    }
    
    /**
     * Creates and returns a rotation matrix.
     * 
     * @param angleRad The angle
     * @return The matrix
     */
    public static Matrix4f rotateZ(double angleRad)
    {
        Matrix4f result = identity();
        result.rotZ((float)angleRad);
        return result;
    }
    
    /**
     * Creates a matrix that describes a rotation of the given
     * angle around the given axis
     * 
     * @param axis The axis of rotation
     * @param angleRad The rotation angle, in radians
     * @return The resulting rotation matrix
     */
    public static Matrix4f rotate(Vector3f axis, double angleRad)
    {
        Matrix4f result = new Matrix4f();
        result.set(new AxisAngle4f(axis, (float)angleRad));
        return result;
    }
    
    /**
     * Creates and returns a translation matrix.
     * 
     * @param t The translation
     * @return The matrix
     */
    public static Matrix4f translate(Tuple3f t)
    {
        return translate(t.x, t.y, t.z);
    }
    
    /**
     * Creates and returns a translation matrix.
     * 
     * @param dx The x-translation
     * @param dy The y-translation
     * @param dz The z-translation
     * @return The matrix
     */
    public static Matrix4f translate(float dx, float dy, float dz)
    {
        Matrix4f result = identity();
        result.m03 = dx;
        result.m13 = dy;
        result.m23 = dz;
        return result;
    }
    
    /**
     * Creates and returns an identity matrix
     * 
     * @return An identity matrix
     */
    public static Matrix4f identity()
    {
        Matrix4f result = new Matrix4f();
        result.setIdentity();
        return result;
    }
    
    
    /**
     * Returns the product of the given matrices
     * 
     * @param matrices The matrices
     * @return The product of the matrices
     */
    public static Matrix4f mul(Matrix4f ... matrices)
    {
        Matrix4f result = identity();
        for (Matrix4f m : matrices)
        {
            result.mul(m);
        }
        return result;
    }

    /**
     * Write the contents of the given matrix into the given buffer
     * in column-major order and returns it. It is assumed that the 
     * given buffer has enough space for the matrix. If the given 
     * buffer is <code>null</code>, a new direct buffer will be  
     * created and returned.
     * 
     * @param matrix The matrix
     * @param buffer The buffer to write to
     * @return The given buffer, now containing the matrix data
     */
    public static FloatBuffer writeMatrixToBuffer(
        Matrix4f matrix, FloatBuffer buffer)
    {
        if (buffer == null)
        {
            buffer = BufferUtils.createFloatBuffer(16);
        }
        buffer.put(matrix.m00);
        buffer.put(matrix.m10);
        buffer.put(matrix.m20);
        buffer.put(matrix.m30);
        buffer.put(matrix.m01);
        buffer.put(matrix.m11);
        buffer.put(matrix.m21);
        buffer.put(matrix.m31);
        buffer.put(matrix.m02);
        buffer.put(matrix.m12);
        buffer.put(matrix.m22);
        buffer.put(matrix.m32);
        buffer.put(matrix.m03);
        buffer.put(matrix.m13);
        buffer.put(matrix.m23);
        buffer.put(matrix.m33);
        return buffer;
    }


    
    /**
     * Write the contents of the given buffer into the given matrix
     * in column-major order and returns it. It is assumed that the 
     * given buffer has enough remaining data to fill the matrix. 
     * If the given matrix is <code>null</code>, a new matrix is 
     * created and returned. 
     * 
     * @param buffer The buffer to read from
     * @param matrix The matrix to fill
     * @return The given matrix, now containing the buffer data
     */
    public static Matrix4f writeBufferToMatrix(
        FloatBuffer buffer, Matrix4f matrix)
    {
        if (matrix == null)
        {
            matrix = new Matrix4f();
        }
        matrix.m00 = buffer.get();
        matrix.m10 = buffer.get();
        matrix.m20 = buffer.get();
        matrix.m30 = buffer.get();
        matrix.m01 = buffer.get();
        matrix.m11 = buffer.get();
        matrix.m21 = buffer.get();
        matrix.m31 = buffer.get();
        matrix.m02 = buffer.get();
        matrix.m12 = buffer.get();
        matrix.m22 = buffer.get();
        matrix.m32 = buffer.get();
        matrix.m03 = buffer.get();
        matrix.m13 = buffer.get();
        matrix.m23 = buffer.get();
        matrix.m33 = buffer.get();
        
        return matrix;
    }
    
    /**
     * Private constructor to prevent instantiation
     */
    private MatrixUtils()
    {
    }
    
}



