/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl;

import java.awt.event.MouseEvent;

/**
 * This class contains utility methods for creating 
 * {@link Condition Conditions} that may be applied
 * to MouseEvents.
 */
public class MouseEventConditions
{
    /**
     * Returns a new condition that checks whether mouse events have
     * happened while the CTRL button was down
     *  
     * @return The new condition
     */
    public static Condition<MouseEvent> controlDownCondition()
    {
        return new Condition<MouseEvent>()
        {
            @Override
            public boolean is(MouseEvent t)
            {
                return t.isControlDown();
            }
        };
    }

    /**
     * Returns a new condition that checks whether mouse events have
     * happened while the ALT button was down
     *  
     * @return The new condition
     */
    public static Condition<MouseEvent> altDownCondition()
    {
        return new Condition<MouseEvent>()
        {
            @Override
            public boolean is(MouseEvent t)
            {
                return t.isAltDown();
            }
        };
    }

    /**
     * Returns a new condition that checks whether mouse events have
     * happened while the ALT-GR button was down
     *  
     * @return The new condition
     */
    public static Condition<MouseEvent> altGraphDownCondition()
    {
        return new Condition<MouseEvent>()
        {
            @Override
            public boolean is(MouseEvent t)
            {
                return t.isAltGraphDown();
            }
        };
    }

    /**
     * Returns a new condition that checks whether mouse events have
     * happened while the SHIFT button was down
     *  
     * @return The new condition
     */
    public static Condition<MouseEvent> shiftDownCondition()
    {
        return new Condition<MouseEvent>()
        {
            @Override
            public boolean is(MouseEvent t)
            {
                return t.isShiftDown();
            }
        };
    }

    /**
     * Returns a new condition that checks whether mouse events have
     * happened while the META button was down
     *  
     * @return The new condition
     */
    public static Condition<MouseEvent> metaDownCondition()
    {
        return new Condition<MouseEvent>()
        {
            @Override
            public boolean is(MouseEvent t)
            {
                return t.isMetaDown();
            }
        };
    }
    
    /**
     * Returns a new condition that checks whether mouse events are
     * popup triggers.
     *  
     * @return The new condition
     */
    public static Condition<MouseEvent> popupTriggerCondition()
    {
        return new Condition<MouseEvent>()
        {
            @Override
            public boolean is(MouseEvent t)
            {
                return t.isPopupTrigger();
            }
        };
    }
    
    /**
     * Private constructor to prevent instantiation.
     */
    private MouseEventConditions()
    {
    }
}
