/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.geometry;

import javax.vecmath.Tuple2f;
import javax.vecmath.Tuple3f;

import de.javagl.swogl.SwoglComponent;

/**
 * Interface for the geometry of a {@link SwoglComponent}.<br />
 * <br />
 */
public interface Geometry 
{
    /**
     * Returns the number of triangles in this Geometry
     * 
     * @return The number of triangles in this Geometry
     */
    int getNumTriangles();

    /**
     * Returns the specified vertex index. Three consecutive indices
     * are the indices of the vertices of a single triangle
     * 
     * @param index The number of the index to return
     * @return The index
     */
    int getVertexIndex(int index);

    /**
     * Returns the number of vertices in this Geometry
     * 
     * @return The number of vertices in this Geometry
     */
    int getNumVertices();

    /**
     * Stores the vertex with the given index in the given argument
     * 
     * @param index The index of the vertex
     * @param vertex Will store the vertex
     */
    void getVertex(int index, Tuple3f vertex);

    /**
     * Set the coordinates of the vertex with the specified index
     *  
     * @param index The index of the vertex
     * @param vertex The coordinates the vertex should have
     */
    void setVertex(int index, Tuple3f vertex);

    /**
     * Stores the normal with the given index in the given argument
     * 
     * @param index The index of the vertex whose normal should be obtained
     * @param normal Will store the normal
     */
    void getNormal(int index, Tuple3f normal);

    /**
     * Sets the normal of the vertex with the given index
     * 
     * @param index The index of the vertex whose normal should be set
     * @param normal The normal to set for the vertex
     */
    void setNormal(int index, Tuple3f normal);

    /**
     * Stores the texture coordinates of the vertex with the given
     * index in the given argument
     * 
     * @param index The index of the vertex whose texture coordinates 
     * should be obtained
     * @param texCoord Will store the texture coordinates
     */
    void getTexCoord(int index, Tuple2f texCoord);

    /**
     * Set the texture coordinates of the vertex with the given index
     * 
     * @param index The index of the vertex whose texture coordinates 
     * should be set
     * @param texCoord The texture coordinates to set
     */
    void setTexCoord(int index, Tuple2f texCoord);
    
    /**
     * Set the flag which indicates that the vertex positions
     * of this geometry have been modified, and should be
     * updated by the renderer before the next rendering pass.
     * 
     * @param modified Whether the vertices have been modified
     */
    void setVerticesModified(boolean modified);
    
    /**
     * Returns whether the vertices have been modified
     * 
     * @return Whether the vertices have been modified 
     */
    boolean areVerticesModified();
    
    /**
     * Set the flag which indicates that the texture coordinates
     * of this geometry have been modified, and should be
     * updated by the renderer before the next rendering pass.
     * 
     * @param modified Whether the texture coordinates have been modified
     */
    void setTexCoordsModified(boolean modified);

    /**
     * Returns whether the texture coordinates have been modified
     * 
     * @return Whether the texture coordinates have been modified 
     */
    boolean areTexCoordsModified();
    
}
