/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.layout;

import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;

import javax.swing.JComponent;
import javax.swing.KeyStroke;

import de.javagl.swogl.interaction.Control;

/**
 * Simple implementation of a {@link Control} for the default
 * behavior of a {@link BrowseLayout3D}. 
 */
public class BrowseLayoutControl implements Control
{
    /**
     * The {@link BrowseLayout3D} this control operates on
     */
    private BrowseLayout3D browseLayout;
    
    /**
     * The {@link Hotkey} for activating the browse mode
     */
    private Hotkey hotkey;
    
    /**
     * The listener that will be attached to the component
     */
    class Listener implements MouseWheelListener
    {
        @Override
        public void mouseWheelMoved(MouseWheelEvent e)
        {
            if (isEnabled())
            {
                browseLayout.scrollAbout(e.getWheelRotation());
            }
        }
    }
    /**
     * The listener that will be attached to the component
     */
    private Listener listener = new Listener();
    
    /**
     * Whether this control is currently enabled
     */
    private boolean enabled = true;

    /**
     * Creates a new BrowseLayoutControl for the given {@link BrowseLayout3D}.
     * 
     * @param browseLayout The layout
     */
    public BrowseLayoutControl(final BrowseLayout3D browseLayout)
    {
        this.browseLayout = browseLayout;
        
        hotkey = new Hotkey(
            JComponent.WHEN_IN_FOCUSED_WINDOW,
            KeyStroke.getKeyStroke(KeyEvent.VK_ALT, InputEvent.ALT_MASK),
            KeyStroke.getKeyStroke("released ALT"))
        {
            @Override
            protected void pressedStateChanged(boolean pressed)
            {
                setBrowseModeActivated(pressed);
                browseLayout.setBrowseModeActivated(pressed);
            }
        };
    }
    
    /**
     * Will be called when the "browse mode" is activated. May
     * be overridden by subclasses to get informed when the 
     * browse mode is activated. The default implementation
     * is empty.
     *  
     * @param activated Whether the browse mode is activated
     */
    protected void setBrowseModeActivated(boolean activated)
    {
    }
    
    @Override
    public void attachTo(JComponent component)
    {
        hotkey.attachTo(component);
        component.addMouseWheelListener(listener);
    }
    @Override
    public void detachFrom(JComponent component)
    {
        hotkey.detachFrom(component);
        component.removeMouseWheelListener(listener);
    }
    
    @Override
    public void setEnabled(boolean enabled)
    {
        this.enabled = enabled;
        hotkey.setEnabled(enabled);
    }

    @Override
    public boolean isEnabled()
    {
        return enabled;
    }
    
}
