/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.layout;

import java.util.HashMap;
import java.util.Map;

import javax.vecmath.Matrix4f;
import javax.vecmath.Vector3f;

import de.javagl.swogl.MatrixUtils;
import de.javagl.swogl.SwoglComponent;
import de.javagl.swogl.SwoglContainer;
import de.javagl.swogl.geometry.Box;
import de.javagl.swogl.geometry.Geometry;
import de.javagl.swogl.geometry.GeometryUtils;

/**
 * An implementation of the LayoutManager3D interface that lays out
 * the SwoglComponents in a grid. Double-clicking a component 
 * zooms the component to unit size and centers it. Double-clicking
 * with the right mouse button (or outside of a SwoglComponent)
 * zooms out to make all components visible.
 */
public class GridLayout3D implements LayoutManager3D
{
    /**
     * Time in milliseconds between the animation steps
     */
    private static final int ANIMATION_DELAY = 30;

    /**
     * The speed for the zooming animation
     */
    private static final float ZOOMING_SPEED = 0.075f;
    
    /**
     * The number of rows of this GridLayout3D
     */
    private final int rows;

    /**
     * The number of columns of this GridLayout3D
     */
    private final int columns;

    /**
     * The horizontal gap between components
     */
    private final int hGap;

    /**
     * The vertical gap between components
     */
    private final int vGap;
    
    /**
     * The scaling factor for the components in the
     * grid overview
     */
    private float overviewScale = 1.0f;
    
    /**
     * The translation in the grid overview
     */
    private Vector3f overviewTranslation = new Vector3f();

    /**
     * The scaling factor where the current animation started
     */
    private float sourceScale = 1.0f;
    
    /**
     * The translation where the current animation started
     */
    private Vector3f sourceTranslation = new Vector3f();
    
    /**
     * The scaling factor that the current animation is heading for
     */
    private float targetScale = 1.0f;
    
    /**
     * The translation that the current animation is heading for
     */
    private Vector3f targetTranslation = new Vector3f();

    /**
     * The current scaling factor
     */
    private float currentScale = 1.0f;
    
    /**
     * The current translation
     */
    private Vector3f currentTranslation = new Vector3f();
    
    /**
     * Whether this layout is currently in the overview mode
     */
    private boolean inOverviewMode = true;
    
    /**
     * The current alpha value between 0 and 1, describing the 
     * relative position between the source- and target scaling 
     * and translation.
     * Initialized with positive infinity to indicate an
     * invalid value.  
     */
    private float currentAlpha = 0;
    
    /**
     * The target alpha value between 0 and 1 that the animation
     * is currently heading for.
     */
    private float targetAlpha = 0;

    /**
     * A map from the current SwoglComponents to their position inside
     * the grid
     */
    private Map<SwoglComponent, Vector3f> gridPositions = 
        new HashMap<SwoglComponent, Vector3f>();
    
    /**
     * The ZoomingAnimator for zooming in and out
     */
    private ZoomingAnimator zoomingAnimator;
    
    /**
     * The SwoglContainer this layout operates on
     */
    private SwoglContainer swoglContainer;
    
    /**
     * Creates a new GridLayout3D that lays out the SwoglComponents of 
     * a SwoglContainer in a grid with the given number of rows and
     * columns and with the given spacing between the components.
     * 
     * @param rows The rows of the grid
     * @param columns The columns of the grid.
     * @param hGap The horizontal gap between components
     * @param vGap The vertical gap between components
     */
    public GridLayout3D(int rows, int columns, int hGap, int vGap)
    {
        this.rows = rows;
        this.columns = columns;
        this.hGap = hGap;
        this.vGap = vGap;
    }

    /**
     * Creates a new GridLayout3D that lays out the SwoglComponents of a
     * SwoglContainer in a grid with the given number of rows and
     * columns and with no space between the components.
     * 
     * @param rows The rows of the grid
     * @param columns The columns of the grid.
     */
    public GridLayout3D(int rows, int columns)
    {
        this(rows, columns, 0, 0);
    }

    @Override
    public void setSwoglContainer(SwoglContainer swoglContainer)
    {
        this.swoglContainer = swoglContainer;
        if (zoomingAnimator != null)
        {
            zoomingAnimator.stop();
            zoomingAnimator = null;
        }
        if (swoglContainer != null)
        {
            computeOverview();

            sourceTranslation.set(overviewTranslation);
            sourceScale = overviewScale;
            
            targetTranslation.set(overviewTranslation);
            targetScale = overviewScale;
            
            zoomingAnimator = new ZoomingAnimator();
            Thread animatorThread = new Thread(zoomingAnimator,
            	"GridLayout3DAnimator");
            animatorThread.setDaemon(true);
            animatorThread.start();
        }
    }
    
    /**
     * Compute the overviewScale, overviewTranslation and the gridPositions.
     * Thus, all information that is required for the layout in the 
     * overview.
     */
    private void computeOverview()
    {
        // Compute the real number of rows and columns
        int numComponents = swoglContainer.getSwoglComponentCount();
        int rs = rows;
        int cs = columns;
        if (rs > 0) 
        {
            cs = (numComponents + rs - 1) / rs;
        } 
        else 
        {
            rs = (numComponents + cs - 1) / cs;
        }

        // Compute the maximum size of any SwoglComponent
        float maxW = -Float.MAX_VALUE;
        float maxH = -Float.MAX_VALUE;
        if (numComponents == 0)
        {
            maxW = 1;
            maxH = 1;
        }
        else
        {
            for (int c = 0; c < cs ; c++) 
            {
                for (int r = 0; r < rs ; r++) 
                {
                    int i = r * cs + c;
                    if (i < numComponents) 
                    {
                        SwoglComponent swoglComponent = 
                            swoglContainer.getSwoglComponent(i);
                        Geometry geometry = swoglComponent.getGeometry();
                        Box boundingBox = 
                            GeometryUtils.computeBoundingBox(geometry);
                        maxW = Math.max(maxW, boundingBox.getSizeX());
                        maxH = Math.max(maxH, boundingBox.getSizeY());
                    }
                }
            }
        }
        maxW += hGap;
        maxH += vGap;

        // Compute the positions of the SwoglComponents when they
        // are arranged in a grid
        gridPositions.clear();
        for (int r = 0; r < rs ; r++) 
        {
            for (int c = 0; c < cs ; c++) 
            {
                int i = r * cs + c;
                if (i < numComponents) 
                {
                    float x = c * maxW + maxW / 2;
                    float y = -(r * maxH + maxH / 2);
                    
                    //System.out.println("Component "+i+" at "+r+" "+c+" index "+i+" at "+x+" "+y);
                    
                    SwoglComponent swoglComponent = 
                        swoglContainer.getSwoglComponent(i);
                    gridPositions.put(swoglComponent, new Vector3f(x,y,0));
                }
            }
        }

        // Compute the overviewScale and overviewTranslation, i.e. the
        // scaling and translation that are required to make all 
        // components visible
        float totalW = maxW * cs;
        float totalH = maxH * rs;
        float containerW = swoglContainer.getMainComponent().getWidth();
        float containerH = swoglContainer.getMainComponent().getHeight();
        float scaleX = containerW / totalW;
        float scaleY = containerH / totalH;
        overviewScale = Math.min(scaleX, scaleY);
        float dx = - overviewScale * totalW / 2;
        float dy = overviewScale * totalH / 2;
        overviewTranslation = new Vector3f(dx, dy, 0);

        /*/
        System.out.println("totalW "+totalW);
        System.out.println("totalH "+totalH);
        System.out.println("scaleX "+scaleX);
        System.out.println("scaleY "+scaleY);
        System.out.println("overviewTranslation "+overviewTranslation);
        //*/
    }

    
    
    @Override
    public void doLayout3D()
    {
        computeOverview();

        // If the overview is currently shown, adjust the layout
        // to take into account new components that may have
        // been added
        if (inOverviewMode)
        {
        	targetTranslation.set(overviewTranslation);
        	targetScale = overviewScale;
        	targetAlpha = 1.0f;
        	synchronized(this)
        	{
        		// Trigger the ZoomingAnimator
        		notifyAll();
        	}
        }

        // Compute the current scaling and translation according
        // to the currentAlpha, and compute the transformation
        // matrix that is common for all SwoglComponents
        currentScale = 
            sourceScale + (targetScale - sourceScale) * currentAlpha; 
        currentTranslation.interpolate(
            sourceTranslation, targetTranslation, currentAlpha);
        Matrix4f transform = MatrixUtils.scale(
            currentScale, currentScale, currentScale);
        transform.setTranslation(currentTranslation);

        //System.out.println("Transform\n"+transform);

        // Arrange the SwoglComponents in a grid according to
        // their GridPositions that has been computed above
        int numComponents = swoglContainer.getSwoglComponentCount();
        for (int i=0; i<numComponents; i++)
        {
        	SwoglComponent swoglComponent = 
        	    swoglContainer.getSwoglComponent(i);
        	Matrix4f matrix = new Matrix4f(transform);
        	Vector3f translation = gridPositions.get(swoglComponent);
        	matrix.mul(MatrixUtils.translate(translation));
        	swoglComponent.setTransform(matrix);
        }
    }
    
    
    /**
     * Zoom to the given SwoglComponent. If the given SwoglComponent is 
     * null, then this GridLayout3D will zoom to the overview.
     * 
     * @param swoglComponent The SwoglComponent to zoom to.
     */
    public void zoomTo(SwoglComponent swoglComponent)
    {
        sourceScale = currentScale;
        sourceTranslation.set(currentTranslation);
        
        if (swoglComponent == null)
        {
            inOverviewMode = true;
            targetScale = overviewScale;
            targetTranslation.set(overviewTranslation);
        }
        else
        {
            inOverviewMode = false;
            targetScale = 1.0f;
            Vector3f translation = gridPositions.get(swoglComponent);
            targetTranslation.set(translation);
            targetTranslation.negate();
        }
        
        targetAlpha = 1.0f;
        currentAlpha = 0.0f;
        
        synchronized(this)
        {
            // Trigger the ZoomingAnimator
            notifyAll();
        }
    }
    
    /**
     * The class that performs the zooming animation
     */
    private class ZoomingAnimator implements Runnable
    {
        private boolean stopped = false;
        
        void stop()
        {
            synchronized (GridLayout3D.this)
            {
                stopped = true;
                GridLayout3D.this.notifyAll();
            }
        }
        
        @Override
        public void run()
        {
            while (!stopped)
            {
                if (currentAlpha < targetAlpha)
                {
                    // Just head toward the target alpha
                    float a = currentAlpha + ZOOMING_SPEED;
                    currentAlpha = Math.min(a, targetAlpha);

                    swoglContainer.validate3D();
                    try
                    {
                        Thread.sleep(ANIMATION_DELAY);
                    }
                    catch (InterruptedException e)
                    {
                        Thread.currentThread().interrupt();
                    }
                }
                else
                {
                    // Wait until this ZoomingAnimator is triggered
                    // by a call to zoomTo or a layout operation in
                    // the overview mode
                    synchronized (GridLayout3D.this)
                    {
                        try
                        {
                            GridLayout3D.this.wait();
                        }
                        catch (InterruptedException e)
                        {
                            Thread.currentThread().interrupt();
                        }
                    }
                }
            }
        }
    }



}
