/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.layout;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ActionMap;
import javax.swing.InputMap;
import javax.swing.JComponent;
import javax.swing.KeyStroke;

import de.javagl.swogl.interaction.Control;

/**
 * Package private utility class that provides the functionality of
 * a simple hotkey.
 */
abstract class Hotkey implements Control
{
    /**
     * The condition for the hotkey. Must be one of<br />
     * JComponent.WHEN_FOCUSED<br />
     * JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT<br />
     * JComponent.WHEN_IN_FOCUSED_WINDOW 
     */
    private final int condition;

    /**
     * The keystroke defining when the hotkey is pressed
     */
    private final KeyStroke keyPressed;

    /**
     * The keystroke defining when the hotkey is released
     */
    private final KeyStroke keyReleased;

    /**
     * The key for the InputMap when this hotkey is pressed
     */
    private final Object pressedID;

    /**
     * The key for the InputMap when this hotkey is released
     */
    private final Object releasedID;
    
    /**
     * The key of the InputMap that was assigned to the 'keyPressed'
     * KeyStroke before this hotkey was attached.
     */
    private Object oldPressedID;

    /**
     * The key of the InputMap that was assigned to the 'keyReleased'
     * KeyStroke before this hotkey was attached.
     */
    private Object oldReleasedID;

    /**
     * The Action of the ActionMap that was assigned to the 'pressedID'
     * before this hotkey was attached.
     */
    private Action oldPressedAction;

    /**
     * The Action of the ActionMap that was assigned to the 'releasedID'
     * before this hotkey was attached.
     */
    private Action oldReleasedAction;
    
    /**
     * The Action of the ActionMap that will be assigned to the 'pressedID'
     * when this hotkey was attached.
     */
    private final Action pressedAction;

    /**
     * The Action of the ActionMap that will be assigned to the 'releasedID'
     * when this hotkey was attached.
     */
    private final Action releasedAction;
    
    /**
     * Whether this control is enabled
     */
    private boolean enabled = true;
    
    /**
     * Creates a new Hotkey 
     * 
     * @param condition The condition for the hotkey. Must be one of<br />
     * JComponent.WHEN_FOCUSED<br />
     * JComponent.WHEN_ANCESTOR_OF_FOCUSED_COMPONENT<br />
     * JComponent.WHEN_IN_FOCUSED_WINDOW 
     * @param pressed The keystroke defining when the hotkey is pressed
     * @param released The keystroke defining when the hotkey is released
     */
    Hotkey(int condition, KeyStroke pressed, KeyStroke released)
    {
        this.condition = condition;

        String id = Integer.toHexString(System.identityHashCode(this));
        pressedID = "pressed-"+id;
        releasedID = "released-"+id;
        
        this.keyPressed = pressed;
        this.keyReleased = released;
        
        pressedAction = new AbstractAction()
        {
            private static final long serialVersionUID = 918348500701920280L;

            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (isEnabled())
                {
                    pressedStateChanged(true);
                }
            }
        };
        releasedAction = new AbstractAction()
        {
            private static final long serialVersionUID = 4908423229342248173L;

            @Override
            public void actionPerformed(ActionEvent e)
            {
                if (isEnabled())
                {
                    pressedStateChanged(false);
                }
            }
        };
    }
    
    /**
     * Called when this hotkey was pressed or released.
     * 
     * @param pressed Whether this hotkey was pressed or released
     */
    protected abstract void pressedStateChanged(boolean pressed);

    @Override
    public void setEnabled(boolean enabled)
    {
        this.enabled = enabled;
    }

    @Override
    public boolean isEnabled()
    {
        return enabled;
    }
    
    @Override
    public void attachTo(JComponent component)
    {
        InputMap inputMap = component.getInputMap(condition);
        ActionMap actionMap = component.getActionMap();
        
        oldPressedID = inputMap.get(keyPressed);
        inputMap.put(keyPressed, pressedID);
        
        oldPressedAction = actionMap.get(pressedID);
        actionMap.put(pressedID, pressedAction);
        
        oldReleasedID = inputMap.get(keyReleased);
        inputMap.put(keyReleased, releasedID);

        oldReleasedAction = actionMap.get(releasedID);
        actionMap.put(releasedID, releasedAction);
    }
    
    @Override
    public void detachFrom(JComponent component)
    {
        InputMap inputMap = component.getInputMap(condition);
        ActionMap actionMap = component.getActionMap();

        inputMap.put(keyPressed, oldPressedID);
        actionMap.put(pressedID, oldPressedAction);
        inputMap.put(keyReleased, oldReleasedID);
        actionMap.put(releasedID, oldReleasedAction);
    }
    
}
