/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.rendering;


import java.awt.Component;
import java.awt.Rectangle;
import java.nio.FloatBuffer;

import javax.media.opengl.GL2;
import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLProfile;
import javax.media.opengl.awt.GLCanvas;
import javax.media.opengl.awt.GLJPanel;
import javax.vecmath.Matrix4f;
import javax.vecmath.Point3f;

import de.javagl.swogl.BufferUtils;
import de.javagl.swogl.DefaultPickingRayComputer;
import de.javagl.swogl.PickingRayComputer;
import de.javagl.swogl.SwoglApplication;
import de.javagl.swogl.SwoglComponentHandler;
import de.javagl.swogl.SwoglRenderer;
import de.javagl.swogl.scene.Camera;
import de.javagl.swogl.scene.CameraListener;
import de.javagl.swogl.scene.CameraUtils;
import de.javagl.swogl.scene.DefaultCamera;
import de.javagl.swogl.scene.DefaultView;
import de.javagl.swogl.scene.MutableCamera;
import de.javagl.swogl.scene.View;
import de.javagl.swogl.scene.ViewUtils;

/**
 * Implementation of a {@link SwoglApplication} using JOGL
 */
public class SwoglApplicationJOGL implements SwoglApplication
{
    /**
     * A direct buffer to temporarily store 4 float values
     */
    private static final FloatBuffer tempBuffer4 = 
        BufferUtils.createFloatBuffer(4);
    
    /**
     * The current GL
     */
    private GL2 gl;

    /**
     * The {@link SwoglRenderer} used in this application
     */
    private final SwoglRendererJOGL swoglRenderer;
    
    /**
     * The {@link View} that is used in this application
     */
    private final DefaultView view;
    
    /**
     * The {@link Camera} that is used in this application
     */
    private final MutableCamera camera;
    
    /**
     * The {@link PickingRayComputer} that is used in this application
     */
    private final PickingRayComputer pickingRayComputer;

    /**
     * The rendering component of this application
     */
    private Component canvas;
    
    /**
     * Whether a GLJPanel should be used instead of a GLCanvas
     * (experimental)
     */
    private final boolean useGLJPanel;    
    
    /**
     * The GLEventListener that will be attached to the rendering component
     */
    private GLEventListener glEventListener = new GLEventListener()
    {
        @Override
        public void init(GLAutoDrawable drawable)
        {
            GL2 gl = drawable.getGL().getGL2();
            SwoglApplicationJOGL.this.gl = gl;
            swoglRenderer.setGL(gl);
            gl.setSwapInterval(0);
        }
        
        @Override
        public void display(GLAutoDrawable drawable) 
        {
            render();
        }
        
        @Override
        public void reshape(
            GLAutoDrawable drawable, int x, int y, int width, int height)
        {
            setupView();
        }

        @Override
        public void dispose(GLAutoDrawable arg0)
        {
        }
    };

    
    /**
     * Creates a new SwoglApplicationJOGL
     */
    public SwoglApplicationJOGL()
    {
        this(false);
    }
    
    /**
     * Creates a new SwoglApplicationJOGL.
     * 
     * @param useGLJPanel Whether a GLJPanel should be used
     * instead of a GLCanvas (experimental)
     */
    public SwoglApplicationJOGL(boolean useGLJPanel)
    {
        this.view = new DefaultView();
        this.camera = new DefaultCamera();
        this.camera.setEyePoint(new Point3f(0,0,500));
        this.pickingRayComputer = 
            new DefaultPickingRayComputer(getCamera(), getView());
        
        this.swoglRenderer = new SwoglRendererJOGL();
        this.useGLJPanel = useGLJPanel;
        initialize();
    }
    
    /**
     * Internal initialization
     */
    public void initialize()
    {
        GLProfile profile = GLProfile.get(GLProfile.GL2);
        GLCapabilities capabilities = new GLCapabilities(profile);
        capabilities.setNumSamples(2);
        capabilities.setSampleBuffers(true);
        
        if (useGLJPanel)
        {
            GLJPanel c = new GLJPanel(capabilities);
            c.addGLEventListener(glEventListener);
            canvas = c;
        }
        else
        {
            GLCanvas c = new GLCanvas(capabilities);
            c.addGLEventListener(glEventListener);
            canvas = c;
        }
        
        camera.addCameraListener(new CameraListener()
        {
            @Override
            public void cameraChanged(Camera camera)
            {
                canvas.repaint();
            }
        });
    }
    
    @Override
    public View getView()
    {
        return view;
    }
    
    @Override
    public MutableCamera getCamera()
    {
        return camera;
    }
    
    @Override 
    public PickingRayComputer getPickingRayComputer()
    {
        return pickingRayComputer;
    }
    
    @Override
    public Component getRenderComponent()
    {
        return canvas;
    }
    
    @Override
    public SwoglComponentHandler getSwoglComponentHandler()
    {
        return swoglRenderer;
    }
    
    /**
     * Returns the SwoglRenderer that is used in this application
     * 
     * @return The SwoglRenderer that is used in this application
     */
    protected SwoglRendererJOGL getSwoglRenderer()
    {
        return swoglRenderer;
    }
    
    /**
     * Set up the view according to the current size of the 
     * rendering component
     */
    protected void setupView()
    {
        int width = getRenderComponent().getWidth();
        int height = getRenderComponent().getHeight();
        float aspect = (float)width/height;
        view.setAspect(aspect);
        view.setViewport(new Rectangle(0,0,width,height));
        gl.glViewport(0, 0, width, height);
    }
    
    /**
     * Renders the current scene. This will call {@link #prepareRender()}
     * and {@link #applyView()}, and then let the SwoglRenderer render
     * all SwoglComponents.
     */
    protected void render() 
    {
        prepareRender();
        applyView();
        
        gl.glPushMatrix();
        Matrix4f cameraMatrix = CameraUtils.computeCameraMatrix(camera);
        gl.glLoadMatrixf(RenderingUtils.bufferWith(cameraMatrix));
        swoglRenderer.renderAllSwoglComponents();
        
        gl.glPopMatrix();
    }
    
    /**
     * Prepares the rendering pass
     */
    protected void prepareRender()
    {
        gl.glClearColor(0,0,0,0);
        gl.glClear(GL2.GL_COLOR_BUFFER_BIT | GL2.GL_DEPTH_BUFFER_BIT);
        gl.glEnable(GL2.GL_DEPTH_TEST);
        gl.glShadeModel (GL2.GL_SMOOTH);
        
        float ambient[] = { 0.4f, 0.4f, 0.4f, 0.4f };
        float specular[] = { 0.4f, 0.4f, 0.4f, 0.4f };
        float shininess[] = { 80.0f, 0,0,0};
        float lightPosition[] = { -1000.0f, 1000.0f, 1000.0f, 1.0f };

        gl.glEnable(GL2.GL_LIGHTING);
        gl.glEnable(GL2.GL_LIGHT0);

        gl.glMaterialfv(GL2.GL_FRONT_AND_BACK, GL2.GL_AMBIENT, bufferWith(ambient));
        gl.glMaterialfv(GL2.GL_FRONT_AND_BACK, GL2.GL_SPECULAR, bufferWith(specular));
        gl.glMaterialfv(GL2.GL_FRONT_AND_BACK, GL2.GL_SHININESS, bufferWith(shininess));
        gl.glLightfv(GL2.GL_LIGHT0, GL2.GL_POSITION, bufferWith(lightPosition));

    }
    
    /**
     * Apply the current view state to GL.
     */
    protected void applyView()
    {
        gl.glMatrixMode(GL2.GL_PROJECTION);
        Matrix4f projectionMatrix = ViewUtils.computeProjectionMatrix(camera.getFovY(), view);
        gl.glLoadMatrixf(RenderingUtils.bufferWith(projectionMatrix));
        gl.glMatrixMode(GL2.GL_MODELVIEW);        
    }
    
    /**
     * Writes the given array (which may have not more than 4 elements)
     * into a direct buffer, and returns the direct buffer. This method
     * will always return the same buffer instance.
     * 
     * @param data The data to write into the buffer
     * @return The buffer
     */
    private static FloatBuffer bufferWith(float data[])
    {
        tempBuffer4.put(data);
        tempBuffer4.rewind();
        return tempBuffer4;
    }
    

}
