/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.rendering;
import static org.lwjgl.opengl.GL11.*;
import static org.lwjgl.opengl.GL11.GL_CLAMP;
import static org.lwjgl.opengl.GL11.GL_ENABLE_BIT;
import static org.lwjgl.opengl.GL11.GL_LINEAR;
import static org.lwjgl.opengl.GL11.GL_MODELVIEW;
import static org.lwjgl.opengl.GL11.GL_ONE_MINUS_SRC_ALPHA;
import static org.lwjgl.opengl.GL11.GL_REPLACE;
import static org.lwjgl.opengl.GL11.GL_RGBA;
import static org.lwjgl.opengl.GL11.GL_SRC_ALPHA;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_2D;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_BIT;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_ENV;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_ENV_MODE;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_MAG_FILTER;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_MIN_FILTER;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_WRAP_S;
import static org.lwjgl.opengl.GL11.GL_TEXTURE_WRAP_T;
import static org.lwjgl.opengl.GL11.GL_TRIANGLES;
import static org.lwjgl.opengl.GL11.GL_UNPACK_ALIGNMENT;
import static org.lwjgl.opengl.GL11.GL_UNSIGNED_BYTE;
import static org.lwjgl.opengl.GL11.glBegin;
import static org.lwjgl.opengl.GL11.glBindTexture;
import static org.lwjgl.opengl.GL11.glBlendFunc;
import static org.lwjgl.opengl.GL11.glColor3f;
import static org.lwjgl.opengl.GL11.glDeleteTextures;
import static org.lwjgl.opengl.GL11.glDisable;
import static org.lwjgl.opengl.GL11.glEnable;
import static org.lwjgl.opengl.GL11.glEnd;
import static org.lwjgl.opengl.GL11.glGenTextures;
import static org.lwjgl.opengl.GL11.glMatrixMode;
import static org.lwjgl.opengl.GL11.glMultMatrix;
import static org.lwjgl.opengl.GL11.glNormal3f;
import static org.lwjgl.opengl.GL11.glPixelStorei;
import static org.lwjgl.opengl.GL11.glPopAttrib;
import static org.lwjgl.opengl.GL11.glPopMatrix;
import static org.lwjgl.opengl.GL11.glPushAttrib;
import static org.lwjgl.opengl.GL11.glPushMatrix;
import static org.lwjgl.opengl.GL11.glTexCoord2f;
import static org.lwjgl.opengl.GL11.glTexEnvi;
import static org.lwjgl.opengl.GL11.glTexImage2D;
import static org.lwjgl.opengl.GL11.glTexParameteri;
import static org.lwjgl.opengl.GL11.glVertex3f;
import static org.lwjgl.opengl.GL12.GL_BGRA;

import java.awt.image.BufferedImage;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.IntBuffer;
import java.util.HashMap;
import java.util.Map;

import javax.vecmath.Matrix4f;
import javax.vecmath.Point3f;
import javax.vecmath.TexCoord2f;
import javax.vecmath.Tuple2f;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

import org.lwjgl.opengl.GL11;

import de.javagl.swogl.BufferUtils;
import de.javagl.swogl.SwoglComponent;
import de.javagl.swogl.SwoglRenderer;
import de.javagl.swogl.Texture;
import de.javagl.swogl.geometry.Geometry;
import de.javagl.swogl.geometry.GeometryUtils;

/**
 * Implementation of a {@link SwoglRenderer} that uses LWJGL.
 */
public class SwoglRendererLWJGL extends AbstractSwoglRenderer 
    implements SwoglRenderer
{
    /**
     * The map from Textures to GL texture IDs
     */
    private final Map<Texture, Integer> textureIDs;
    
    /**
     * A map from Textures to buffers which are used for the update
     */
    private final Map<Texture, IntBuffer> textureBuffers;
    
    /**
     * Creates a new SwoglRendererLWJGL
     */
    public SwoglRendererLWJGL()
    {
        textureIDs = new HashMap<Texture, Integer>();
        textureBuffers = new HashMap<Texture, IntBuffer>();
    }
    

    @Override
    protected void handleSwoglComponentInternal(SwoglComponent swoglComponent)
    {
        Texture texture = swoglComponent.getTexture();
        initTexture(texture);
    }
    
    private void initTexture(Texture texture)
    {
        int textureID = glGenTextures();
        textureIDs.put(texture, textureID);
        
        glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
        
        glBindTexture(GL_TEXTURE_2D, textureID);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        
        updateTextureInternal(texture);
    }
    
    
    @Override
    protected void updateTextureInternal(Texture texture)
    {
        BufferedImage image = texture.getImage();
        int w = image.getWidth();
        int h = image.getHeight();

        IntBuffer textureBuffer = textureBuffers.get(texture);
        if (textureBuffer == null)
        {
            ByteBuffer buffer = BufferUtils.createByteBuffer(w*h*4);
            textureBuffer = buffer.order(ByteOrder.LITTLE_ENDIAN).asIntBuffer();
            textureBuffers.put(texture, textureBuffer);
        }
        ImageUtils.writeImageToBuffer(image, textureBuffer);
        textureBuffer.rewind();

        int textureID = textureIDs.get(texture);
        glBindTexture(GL_TEXTURE_2D, textureID);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w,
            h, 0, GL_BGRA, GL_UNSIGNED_BYTE, textureBuffer);
    }
    
    

    @Override
    protected void releaseSwoglComponentInternal(final SwoglComponent swoglComponent)
    {
        Texture texture = swoglComponent.getTexture();
        releaseTexture(texture);
    }

    private void releaseTexture(Texture texture)
    {
        int textureID = textureIDs.get(texture);
        glDeleteTextures(textureID);
        textureIDs.remove(texture);
    }
    


    @Override
    protected void updateVerticesInternal(Geometry geometry)
    {
        GeometryUtils.updateNormals(geometry);
    }
    
    @Override
    public void renderSwoglComponentInternal(SwoglComponent swoglComponent)
    {
        Geometry geometry = swoglComponent.getGeometry();
        Texture texture = swoglComponent.getTexture();
        
        glMatrixMode(GL_MODELVIEW);        
        glPushMatrix();
        
        //paintGrid();
        
        Matrix4f transformMatrix = swoglComponent.getTransform();
        glMultMatrix(RenderingUtils.bufferWith(transformMatrix));

        glPushAttrib(
            GL_ENABLE_BIT |
            GL_TEXTURE_BIT); 
        
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
        glEnable(GL_TEXTURE_2D);
        glEnable(GL_COLOR_MATERIAL);
        int textureID = textureIDs.get(texture);
        glBindTexture(GL_TEXTURE_2D, textureID);
        
        Tuple3f vertex = new Point3f();
        Tuple3f normal = new Vector3f();
        Tuple2f texCoords = new TexCoord2f();
        
        glBegin(GL_TRIANGLES);
        for (int i=0; i<geometry.getNumTriangles(); i++)
        {
            for (int j=0; j<3; j++)
            {
                GeometryUtils.getVertex(
                    geometry, i, j,  vertex, normal, texCoords);
                
                glTexCoord2f(texCoords.x, texCoords.y);
                glNormal3f(normal.x, normal.y, normal.z);
                glVertex3f(vertex.x, vertex.y, vertex.z);
            }
        }
        glEnd();
        glPopMatrix();
        
        glPopAttrib();
    }
    
    /**
     * Paints a simple grid for debugging
     */
    private void paintGrid()
    {
        glDisable(GL11.GL_LIGHTING);
        glBegin(GL11.GL_LINES);
        for (int y=-1000; y<1000; y++)
        {
            if (y%10==0)
            {
                if (y%100==0)
                {
                    glColor3f(1,1,1);
                }
                else
                {
                    glColor3f(0.5f,0.5f,0.5f);
                }
                glVertex3f(   0,   y,   0);
                glVertex3f(1000,   y,   0);
            }
        }
        glEnd();
        glEnable(GL11.GL_LIGHTING);
    }
    

}
