/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.scene;

import javax.vecmath.Matrix4f;
import javax.vecmath.Point3f;
import javax.vecmath.Tuple3f;
import javax.vecmath.Vector3f;

import de.javagl.swogl.MatrixUtils;

/**
 * A class containing utility methods for {@link Camera} operations.
 */
public class CameraUtils
{
    /**
     * Adjust the distance between the eye point and the view point. 
     * This method will move the eye point (along the line through 
     * the current eye point and the view point) so that an object 
     * with the given height is completely visible. <br />
     * <br />
     * This method is mainly intended to move the eye point of a camera
     * from its initial configuration along the negative z-axis to a 
     * position where a rectangle with the specified height is 
     * completely visible.<br />
     * <br />
     * Calling this method with a camera pointing along the negative
     * z-axis and the height of the rendering component has the effect
     * of showing the x-y-plane in its actual size (in pixels).
     * 
     * @param camera The camera
     * @param height The height that should be visible
     */
    public static void adjustDistance(MutableCamera camera, float height)
    {
        double fovYrad = Math.toRadians(camera.getFovY());
        float distance = (float)((height / 2) / Math.tan(fovYrad / 2));
        Point3f viewPoint = camera.getViewPoint();
        Point3f eyePoint = camera.getEyePoint();
        Vector3f direction = new Vector3f();
        direction.sub(eyePoint, viewPoint);
        direction.normalize();
        direction.scale(distance);
        eyePoint.add(viewPoint, direction);
        camera.setEyePoint(eyePoint);
    }
    
    
    /**
     * Returns the matrix for the given camera. The orientation described
     * by the matrix will be taken from the view point, eye point and up
     * vector, and the translation will be computed from the eye point. 
     * 
     * @param camera The camera
     * @return The camera matrix
     */
    public static Matrix4f computeCameraMatrix(Camera camera)
    {
        Point3f eye = camera.getEyePoint();
        Point3f center = camera.getViewPoint();
        Vector3f up = camera.getUpVector();
        Matrix4f lookAtMatrix = computeLookAtMatrix(eye, center, up);

        Vector3f negEye = new Vector3f(eye);
        negEye.negate();
        Matrix4f translationMatrix = MatrixUtils.translate(negEye);

        Matrix4f cameraMatrix = MatrixUtils.mul(
            lookAtMatrix, translationMatrix);
        return cameraMatrix;
    }

    /**
     * Computes a look-at-transformation matrix
     * 
     * @param eye The eye position
     * @param center The view center
     * @param up The up vector
     * @return The matrix
     */
    private static Matrix4f computeLookAtMatrix(
        Tuple3f eye, Tuple3f center, Tuple3f up)
    {
        Matrix4f m = new Matrix4f();

        Vector3f forward = new Vector3f();
        Vector3f side = new Vector3f();
        Vector3f upV = new Vector3f(up);

        forward.sub(center, eye);
        
        forward.normalize();
        side.cross(forward, upV);
        side.normalize();
        upV.cross(side, forward);

        m.setIdentity();
        
        m.m00 = side.x;
        m.m01 = side.y;
        m.m02 = side.z;

        m.m10 = upV.x;
        m.m11 = upV.y;
        m.m12 = upV.z;

        m.m20 = -forward.x;
        m.m21 = -forward.y;
        m.m22 = -forward.z;

        return m;
    }
    
    /**
     * Private constructor to prevent instantiation
     */
    private CameraUtils()
    {
    }
}
