/*
 * Swogl - Swing meets OpenGL
 * 
 * Copyright 2007-2011 Marco Hutter - http://swogl.javagl.de
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

package de.javagl.swogl.scene;

import javax.vecmath.Matrix4f;

/**
 * A class containing utility methods for {@link View} operations.
 */
public class ViewUtils
{
    /**
     * Returns the projection matrix for the given parameters 
     * 
     * @param fovYdeg The FOV (field-of-view) in y-direction, in degrees
     * @param view The view
     * @return The camera matrix
     */
    public static Matrix4f computeProjectionMatrix(float fovYdeg, View view)
    {
        return computeProjectionMatrix(
            fovYdeg, view.getAspect(), 
            view.getNearClippingPlane(), view.getFarClippingPlane());
    }

    /**
     * Creates a projection matrix with the given parameters.
     * 
     * @param fovy The FOV in y-direction, in degrees
     * @param aspect The aspect ratio
     * @param zNear The near clipping plane.
     * @param zFar The far clipping plane
     * @return The matrix m, filled appropriately
     */
    private static Matrix4f computeProjectionMatrix(
        float fovy, float aspect, float zNear, float zFar)
    {
        Matrix4f m = new Matrix4f();
        float deltaZ = zFar - zNear;
        float radians = (float)Math.toRadians(fovy / 2);
        float sine = (float)Math.sin(radians);
        if ((deltaZ == 0) || (sine == 0) || (aspect == 0))
        {
            return m;
        }
        float cotangent = (float)Math.cos(radians) / sine;

        m.setIdentity();

        m.m00 = cotangent / aspect;
        m.m11 = cotangent;
        m.m22 = -(zFar + zNear) / deltaZ;
        m.m32 = -1;
        m.m23 = -2 * zNear * zFar / deltaZ;
        m.m33 = 0;
        
        return m;
    }

    /**
     * Private constructor to prevent instantiation
     */
    private ViewUtils()
    {
    }
}
